% generate Figure 3.25 in Cellula Automata
% simulate time evolution of Rule 126 for each of the 1024 initial
% configurations of 10 cells wide with preiodic boundary conditions

% plot initial configuration against final configuration

function [states] = Rule126_dylan(ncells,t)

if nargin<2
    t = 2^ncells;
end

tic;

% N_Init initial number of states, in our case 10

states = zeros(t+1,2^ncells); % rows are time points, columns are evolved states corresponding to the jth initial state
states(1,:) = 0:2^ncells-1;

% for i = 2:t+1 % loop over time
limitcycle = 0; i = 1;
while i <= t %~limitcycle && i <= t
    i = i + 1;
    for j = 1:2^ncells  % loop over initial states

        R = zeros(2,ncells+2); % add boundary points
        R(1,2:end-1) = decimal2binary(states(i-1,j),ncells); %initial state
        R(1,1) = R(1,end-1); R(1,end) = R(1,2); % periodic boundary conditions

        for k=2:ncells+1 % loop over interior points

            if R(1,k-1)==1 &&  R(1,k)==1 && R(1,k+1)==1;  % define rule 30

                R(2,k) = 0;

            elseif R(1,k-1)==1 &&  R(1,k)==1 && R(1,k+1)==0;

                R(2,k) = 1;

            elseif R(1,k-1)==1 &&  R(1,k)==0 && R(1,k+1)==1;

                R(2,k) = 1;

            elseif R(1,k-1)==1 &&  R(1,k)==0 && R(1,k+1)==0;

                R(2,k) = 1;

            elseif R(1,k-1)==0 &&  R(1,k)==1 && R(1,k+1)==1;

                R(2,k) = 1;

            elseif R(1,k-1)==0 &&  R(1,k)==1 && R(1,k+1)==0;

                R(2,k) = 1;

            elseif R(1,k-1)==0 &&  R(1,k)==0 && R(1,k+1)==1;

                R(2,k) = 1;

            elseif R(1,k-1)==0 &&  R(1,k)==0 && R(1,k+1)==0;

                R(2,k) = 0;
            end

        end

        R(2,1) = R(2,ncells+1); % add periodic points
        R(2,ncells+2) = R(2,2); % add periodic points
        
        states(i,j) = binary2decimal(R(2,2:end-1));
    end
    limitcycle = ismember(states(i,:),states(1:i-1,:),'rows');
end

states = states(1:i,:);

toc;

return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation to generate Figure 3.25
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ncells = 10;

[states] = Rule126_dylan(ncells);

figure
plot(states(1,:),states(end-1,:),'.k')
xlabel('Initial Configurations')
ylabel('Final Configurations')
axis([0 1023 0 1023])

for i = [2 7:19]; figure; plot(states(1,:),states(i,:),'.k'); axis([0 1023 0 1023]); title(['Rule 126, Size 10, ' num2str(i-1) ' steps']); xlabel('Initial Configurations'); ylabel('Final Configurations'); end