% generate Figure 3.27 in Cellula Automata
% simulate time evolution of Rule 30 for each of the 1024 initial
% configurations of 15 cells wide with reflecting boundary points

% plot initial configuration against final configuration

function [Entr] = Rule30_entr_dylan(NTime, N_Init)

tic;

% NTime maximum number of time steps
% N_Init initial number of states, in our case 15

R_Init = R_Init_Rule30(N_Init); % call subroutine to initialise evolution

Max_Entr = N_Init; %Max_Entr = -log2(1/2^N_Init)

Entr = zeros(1, NTime); % initialise entropy

Entr(1) = Max_Entr;


for t = 2:NTime % work out entropy for each time evolution less than NTime

    R_End  = zeros(2^N_Init,N_Init+2);

    for i = 1:2^N_Init  % loop over R_init

        R = zeros(t,N_Init+2); % add boundary points
        R(1,:) = R_Init(i,:); %initial state

        for j = 2:t % loop over time

            for k=2:N_Init+1 % loop over interior points

                if R(j-1,k-1)==1 &&  R(j-1,k)==1 && R(j-1,k+1)==1;  % define rule 30

                    R(j,k) = 0;

                elseif R(j-1,k-1)==1 &&  R(j-1,k)==1 && R(j-1,k+1)==0;

                    R(j,k) = 0;

                elseif R(j-1,k-1)==1 &&  R(j-1,k)==0 && R(j-1,k+1)==1;

                    R(j,k) = 0;

                elseif R(j-1,k-1)==1 &&  R(j-1,k)==0 && R(j-1,k+1)==0;

                    R(j,k) = 1;

                elseif R(j-1,k-1)==0 &&  R(j-1,k)==1 && R(j-1,k+1)==1;

                    R(j,k) = 1;

                elseif R(j-1,k-1)==0 &&  R(j-1,k)==1 && R(j-1,k+1)==0;

                    R(j,k) = 1;

                elseif R(j-1,k-1)==0 &&  R(j-1,k)==0 && R(j-1,k+1)==1;

                    R(j,k) = 1;

                elseif R(j-1,k-1)==0 &&  R(j-1,k)==0 && R(j-1,k+1)==0;

                    R(j,k) = 0;
                end

            end

            %R(j,1) = R(j,2); % add reflecting points
            %R(j,N_Init+2) = R(j,N_Init+1); % add reflecting points
            
            R(j,1) = R(j,N_Init+1); % add periodic points
            R(j,N_Init+2) = R(j,2); % add periodic points

        end

        R_End(i,:) = R(t,:);

    end

    % map R_end to R_init

    % Ind_Init = 1:2^N_Init;

%     for ii = 1:2^N_Init
% %         jj=1;
% %         while Ind_End(ii)==0 % compare while Ind_End has not been matched to an initial configuration
% %             if  R_End(ii,:) == R_Init(jj,:);
% %                 Ind_End(ii) = jj;
% %             else
% %                 jj=jj+1;
% %             end
% %         end
%     end

% define entropy

freq = hist(bin2dec(int2str(R_End(:,2:end-1))),0:(2^N_Init-1)); % New line of code that converts CA results to integer indices with a binary to digital conversion and then bins the indices
N_Final = 2^N_Init;
prob = freq/N_Final;

log2_prob = zeros(1, 2^N_Init);

for kk=1:2^N_Init
    if prob(kk)>0
        log2_prob(kk) = log2(prob(kk));
    else
        log2_prob(kk) = 0;
    end
end

Entr(t) = -prob*(log2_prob)';

end

toc;

return;
    
% define subroutine to construct initial configurations
% essentially one is defining sample sapce for binomial random variable
    
function R_Init = R_Init_Rule30(N_Init)
if N_Init==1, R_Init = [1; 0];return;end

R_Init = [1; 0];

for i=2:N_Init
    R_Init = [ones(2^(i-1),1) R_Init; zeros(2^(i-1),1) R_Init];
end

% % add reflecting points
% 
% R_Init = [R_Init(:,1) R_Init R_Init(:,end)];

% add periodic points

R_Init = [R_Init(:,end) R_Init R_Init(:,1)];


return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation to generate Figure 3.27
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NTime = 100;
N_Init = 15;

[Entr] = Rule30_entr_dylan(NTime, N_Init);

figure
Time=0:NTime-1;
plot(Time,Entr/Entr(1),'Color','k')
xlabel('Time Steps')
ylabel('Fraction of Maximal Entropy')